from searcHPV.generate_alignment import *
import subprocess
from searcHPV.general import *

#####################
#pipeline of alignment
#Function:
#1. customize reference genome
#2. generate alignment bash script
#3. run alignment script
#fq1: paired-end raw sequencing data, fastq1
#fq2: paired-end raw sequencing data, fastq2
#humRef: human reference genome
#outputDir: output directory
#rmdup: = True: remove duplicate = False : don't remove duplicate, default = TRUE
#multi: if fastq file is in gz format: default = True
def alignment(fq1, fq2, humRef, virRef, outputDir, rmdup = True, gz = True):
    #make output dir
    outputDir = os.path.abspath(outputDir)
    mkdir(outputDir)
    scriptDir = f'{outputDir}/alignment'
    mkdir(scriptDir)

    #catenate humRef and virRef
    ref = catRef(humRef,virRef, outputDir)

    #index humRef and virRef
    indexFile = scriptDir + "/index.sh"
    indexRef(indexFile,humRef,virRef,ref,outputDir)

    #generate alignment bash

    alignmentFile = scriptDir + "/orignal.alignment.sh"
    indelFile = scriptDir + "/indel.alignment.sh"
    generate_alignment_bash(alignmentFile,ref,fq1,fq2,scriptDir,gz)
    generate_indel_alignment_bash(indelFile,ref,scriptDir)
    check_file(alignmentFile)
    check_file(indelFile)

    #rmdup for alignment results
    
    bashFile = scriptDir + f"/alignment.sh"
    if not rmdup:
    ##generate indel alignment bash file
        with open(bashFile,'w') as output:
            output.write(f'''#!/bin/bash
bash {indexFile};
bash {alignmentFile};
bash {indelFile};''')
    else:
        generate_mkdup_bash(indelFile,scriptDir)


        with open(bashFile,'w') as output:
            output.write(f'''#!/bin/bash
bash {indexFile};
bash {alignmentFile};
bash {indelFile};''')

    #run scripts
    check_file(bashFile)
    os.system(f'chmod +x {bashFile}')
    subprocess.call(bashFile)



import subprocess
import os
from searcHPV.generate_call_fusion_virus import *
from searcHPV.selection_contig_call_virus_insertion import *

##############
#Fuction: run pipeline for hpv fusion points calling
#humRef: human reference genome
#virRef: virus reference genome
#out_dir: output directory for seacHPV
#n: poly(n), n*d(A/T/C/G), will report low confidence if contig contains poly(n)
def virus_fusion(humRef,virRef,out_dir,n):
    script_map = mapToRef(out_dir)
    script_mapHg = mapToHgRef(out_dir,humRef)
    script_mapVir = mapToVirRef(out_dir,virRef)
    os.system(f'chmod +x {script_map}')
    subprocess.call(script_map)
    os.system(f'chmod +x {script_mapHg}')
    subprocess.call(script_mapHg)
    os.system(f'chmod +x {script_mapVir}')
    subprocess.call(script_mapVir)

    count_sr(out_dir)
    contigDic = read_mapping_info(out_dir)
    combinedContigDic = cal_virus_ins(contigDic,out_dir)
    selectedAllContig = select_contig(combinedContigDic)
    siteConfidence = siteConf(out_dir)
    # print(siteConfidence.keys())
    # print(selectedAllContig.keys())
    filteredSelectedContig = filter_res(selectedAllContig,siteConfidence)
    write_to_file(filteredSelectedContig,siteConfidence,out_dir,n)



import argparse 
from searcHPV.alignment import *
from searcHPV.genome_fusion import *
from searcHPV.assemble import *
from searcHPV.virus_fusion import *

def main(): 

#pass the arguments from commline line
    parser = argparse.ArgumentParser(prog ='searcHPV', 
                                     description ='An HPV intgegration detection tool for targted capture sequencing') 
  
    parser.add_argument('-fastq1', type = str, dest= "fq1",
                        help ='sequencing data: fastq/fq.gz file', required=True,
                        )
    parser.add_argument('-fastq2', type = str, dest = 'fq2',
                        help ='sequencing data: fastq/fq.gz file', required=True,
                        )
    parser.add_argument('-humRef', type = str, dest = 'humRef',
                        help ='human reference genome: fasta file', required=True,
                        )
    parser.add_argument('-virRef', type = str, dest = 'virRef',
                        help ='HPV reference genome: fasta file', required=True,
                        )
    parser.add_argument('-window', type = int, default=300, dest = 'window',
                        help ='the length of region searching for informative reads, default=300', 
                        )
    parser.add_argument('-clusterWindow', type = int, default=100, dest = 'clusterWindow',
                        help ='the length of window of clustering integration sites, default=100', 
                        )
    parser.add_argument('-gz', action ='store_const', const = True,
                        default = False, dest ='gz', 
                        help ="if fastq files are in gz format")

    parser.add_argument('-poly(dn)', type = int, default=80, dest = 'n',
                        help ='poly(n), n*d(A/T/C/G), will report low confidence if contig contains poly(n), default=80', 
                        )
    
    parser.add_argument('-output', type = str, default="./",dest = 'outputDir',
                        help ='output directory, default "./"', 
                        )
                          
    parser.add_argument('-alignment', action ='store_const', const = True,
                        default = False, dest ='alignment', 
                        help ="run the alignment step, step1")

    parser.add_argument('-genomeFusion', action ='store_const', const = True, 
                        default = False, dest ='genomeFusion', 
                        help ="call the genome fusion points, step2")

    parser.add_argument('-assemble', action ='store_const', const = True, 
                        default = False, dest ='assemble', 
                        help ="local assemble for each integration event, step3")

    parser.add_argument('-hpvFusion', action ='store_const', const = True, 
                        default = False, dest ='hpvFusion', 
                        help ="call the HPV fusion points, step4")
    



    args = parser.parse_args() 
    
  
    if args.alignment:
        alignment(fq1 = args.fq1, fq2 = args.fq2, humRef = args.humRef, virRef = args.virRef, outputDir = args.outputDir, gz = args.gz)
    elif args.genomeFusion: 
        genomeFusion(args.clusterWindow,args.outputDir,args.virRef)
    elif args.assemble: 
        #check result from genomeFusion
        assemble(args.fq1, args.fq2, args.outputDir,args.virRef,args.gz,args.window)
    elif args.hpvFusion:
        #check result from assemble
        virus_fusion(args.humRef,args.virRef,args.outputDir,args.n)
    else:
        alignment(fq1 = args.fq1, fq2 = args.fq2, humRef = args.humRef, virRef = args.virRef, outputDir = args.outputDir, gz = args.gz)
        genomeFusion(args.clusterWindow,args.outputDir,args.virRef)
        assemble(args.fq1, args.fq2, args.outputDir,args.virRef,args.gz,args.window)
        virus_fusion(args.humRef,args.virRef,args.outputDir,args.n)
  
    